@@ -60,15 +60,14 @@ cache_engines = new_defaults()
 #' \code{engine_output(options, out = LIST)} where \code{LIST} is a list that
 #' has the same structure as the output of \code{evaluate::evaluate()}. In this
 #' case, the arguments \code{code} and \code{extra} are ignored, and the list is
-#' passed to an internal function \code{knitr:::wrap()} to return a character
-#' vector of final output.
+#' passed to \code{knitr::sew()} to return a character vector of final output.
 #' @param options A list of chunk options. Usually this is just the object
 #'   \code{options} passed to the engine function; see
 #'   \code{\link{knit_engines}}.
-#' @param code Source code of the chunk, to which the output hook
-#'   \code{source} is applied, unless the chunk option \code{echo} is \code{FALSE}.
-#' @param out Text output from the engine, to which the hook \code{output}
-#'   is applied, unless the chunk option \code{results} is \code{'hide'}
+#' @param code Source code of the chunk, to which the output hook \code{source}
+#'   is applied, unless the chunk option \code{echo} is \code{FALSE}.
+#' @param out Text output from the engine, to which the hook \code{output} is
+#'   applied, unless the chunk option \code{results} is \code{'hide'}
 #' @param extra Any additional text output that you want to include.
 #' @return A character string generated from the source code and output using
 #'   the appropriate output hooks.
@@ -80,27 +79,22 @@ cache_engines = new_defaults()
 #' # expert use only
 #' engine_output(opts_chunk$merge(list(engine = 'python')), out = list(structure(list(src = '1 + 1'), class = 'source'), '2'))
 engine_output = function(options, code, out, extra = NULL) {
-  if (missing(code) && is.list(out)) return(unlist(wrap(out, options)))
+  if (missing(code) && is.list(out)) return(unlist(sew(out, options)))
   if (!is.logical(options$echo)) code = code[options$echo]
   if (length(code) != 1L) code = one_string(code)
   if (options$engine == 'sas' && length(out) > 1L && !grepl('[[:alnum:]]', out[2]))
     out = tail(out, -3L)
   if (length(out) != 1L) out = one_string(out)
   out = sub('([^\n]+)$', '\\1\n', out)
-  # replace the engine names for markup later, e.g. ```Rscript should be ```r
-  options$engine = switch(
-    options$engine, mysql = 'sql', node = 'javascript', psql = 'sql', Rscript = 'r',
-    options$engine
-  )
   if (options$engine == 'stata') {
-    out = gsub('\n+running.*profile.do', '', out)
-    out = sub('...\n+', '', out)
-    out = sub('\n. \nend of do-file\n', '', out)
+    out = gsub('\n+running.*profile\\.do', '', out)
+    out = sub('\\.\\.\\.\n+', '', out)
+    out = sub('\n\\. \nend of do-file\n', '', out)
   }
   one_string(c(
     if (length(options$echo) > 1L || options$echo) knit_hooks$get('source')(code, options),
     if (options$results != 'hide' && !is_blank(out)) {
-      if (options$engine == 'highlight') out else wrap.character(out, options)
+      if (options$engine == 'highlight') out else sew.character(out, options)
     },
     extra
   ))
@@ -140,13 +134,14 @@ eng_interpreted = function(options) {
     )
   } else paste(switch(
     engine, bash = '-c', coffee = '-e', groovy = '-e', lein = 'exec -ep',
-    mysql = '-e', node = '-e', octave = '--eval', perl = '-E', psql = '-c',
-    python = '-c', ruby = '-e', scala = '-e', sh = '-c', zsh = '-c', NULL
+    mysql = '-e', node = '-e', octave = '--eval', perl = '-E', php = '-r',
+    psql = '-c', python = '-c', ruby = '-e', scala = '-e', sh = '-c', zsh = '-c',
+    NULL
   ), shQuote(one_string(options$code)))
 
   opts = get_engine_opts(options$engine.opts, engine)
   # FIXME: for these engines, the correct order is options + code + file
-  code = if (engine %in% c('awk', 'gawk', 'sed', 'sas'))
+  code = if (engine %in% c('awk', 'gawk', 'sed', 'sas', 'psql', 'mysql'))
     paste(code, opts) else paste(opts, code)
   cmd = get_engine_path(options$engine.path, engine)
   out = if (options$eval) {
@@ -173,7 +168,86 @@ get_engine_opts = function(opts, engine, fallback = '') {
   opts %n% fallback
 }
 
-get_engine_path = function(path, engine) get_engine_opts(path, engine, engine)
+get_engine_path = function(path, engine, fallback = engine) {
+  get_engine_opts(path, engine, fallback)
+}
+
+# execute an arbitrary command (optionally with arguments)
+# engine.opts = list(command, input, ext, clean, args, args1, args2)
+eng_exec = function(options) {
+  opts = options$engine.opts
+  if (!is.character(cmd <- opts$command %n% options$command)) stop(
+    "The command of the 'exec' engine must be a character string."
+  )
+  cmd = get_engine_path(options$engine.path, options$engine, cmd)
+  input = function(code, file) {
+    write_utf8(code, file)
+    file
+  }
+  if (is.character(i0 <- opts$input))
+    opts$input = function(code, file) input(code, i0)
+  # turn all chunk options into function except 'command'
+  opts = list_fun(opts, setdiff(names(opts), 'command'))
+
+  # default options
+  opts2 = list(
+    ext = identity, input = input, args = function(code, file) {
+      file
+    }, clean = function(file) {
+      unlink(file)
+    }, args1 = function() NULL, args2 = function() NULL,
+    output = function(options, code, output, file) {
+      engine_output(options, code, output)
+    }
+  )
+
+  opts = merge_list(opts2, opts)
+  cmd2 = basename(cmd)  # in case command is a full path
+  ext = opts$ext(cmd2)  # file extension
+  f = wd_tempfile(cmd2, paste0('.', ext))
+  if (is.function(opts$clean)) on.exit(opts$clean(f), add = TRUE)
+  f = opts$input(options$code, f)
+  a = c(opts$args1(), opts$args(options$code, f), opts$args2())
+
+  out = if (options$eval) {
+    if (options$message) message('running: ', paste(c(cmd, a), collapse = ' '))
+    f2 = wd_tempfile(cmd2)  # capture stderr
+    on.exit(unlink(f2), add = TRUE)
+    tryCatch({
+      res = (if (options$error) suppressWarnings else identity)(
+        system2(cmd, shQuote(a), stdout = TRUE, stderr = f2, env = options$engine.env)
+      )
+      # check error in the content run
+      if (!is.null(attr(res, 'status')) && file.exists(f2) && file.size(f2) > 0) {
+        e = readLines(f2) # f2 may not be UTF-8
+        if (!options$error) stop(one_string(e)) else e
+      } else {
+        res
+      }
+    }, error = function(e) {
+        # error in the command run
+        if (!options$error) stop(e)
+        paste('Error in running command', cmd)
+      }
+    )
+  } else ''
+  # chunk option error=FALSE means we need to signal the error
+  if (!options$error && !is.null(attr(out, 'status'))) stop(one_string(out))
+  options = set_lang(options, eng2lang(xfun::sans_ext(cmd2)))
+  opts$output(options, options$code, out, f)
+}
+
+# turn elements of a list into functions: if an element is not a function, make
+# it a function that returns the non-function value
+list_fun = function(x, which = names(x)) {
+  for (i in which) {
+    if (!is.function(v <- x[[i]])) x[[i]] = local({
+      # a trick to avoid R's lazy evaluation (make a copy of v)
+      v2 = v; function(...) v2
+    })
+  }
+  x
+}
 
 ## C, C++, and Fortran (via R CMD SHLIB)
 eng_shlib = function(options) {
@@ -216,7 +290,6 @@ cache_eng_python = function(options) {
 
 ## Rcpp
 eng_Rcpp = function(options) {
-
   sourceCpp = getFromNamespace('sourceCpp', 'Rcpp')
 
   code = one_string(options$code)
@@ -237,7 +310,6 @@ eng_Rcpp = function(options) {
     do.call(sourceCpp, c(list(code = code), opts))
   }
 
-  options$engine = 'cpp' # wrap up source code in cpp syntax instead of Rcpp
   engine_output(options, code, '')
 }
 
@@ -277,17 +349,23 @@ eng_stan = function(options) {
 eng_tikz = function(options) {
   if (!options$eval) return(engine_output(options, options$code, ''))
 
-  lines = read_utf8(options$engine.opts$template %n%
-                    system.file('misc', 'tikz2pdf.tex', package = 'knitr'))
-  i = grep('%% TIKZ_CODE %%', lines)
-  if (length(i) != 1L)
-    stop("Couldn't find replacement string; or the are multiple of them.")
-
-  s = append(lines, options$code, i)  # insert tikz into tex-template
+  lines = read_utf8(
+    options$engine.opts$template %n% system.file('misc', 'tikz2pdf.tex', package = 'knitr')
+  )
+  # add class options to template
+  lines = insert_template(
+    lines, '%% TIKZ_CLASSOPTION %%', options$engine.opts$classoption %n% 'tikz', TRUE
+  )
+  # insert code into preamble
+  lines = insert_template(
+    lines, '%% EXTRA_TIKZ_PREAMBLE_CODE %%', options$engine.opts$extra.preamble, TRUE
+  )
+  # insert tikz code into the tex template
+  s = insert_template(lines, '%% TIKZ_CODE %%', options$code)
   write_utf8(s, texf <- wd_tempfile('tikz', '.tex'))
   on.exit(unlink(texf), add = TRUE)
 
-  ext = tolower(options$fig.ext %n% dev2ext(options$dev))
+  ext = dev2ext(options)
 
   to_svg = ext == 'svg'
   outf = if (to_svg) tinytex::latexmk(texf, 'latex') else tinytex::latexmk(texf)
@@ -301,8 +379,9 @@ eng_tikz = function(options) {
     # dvisvgm needs to be on the path
     # dvisvgm for windows needs ghostscript bin dir on the path also
     if (Sys.which('dvisvgm') == '') tinytex::tlmgr_install('dvisvgm')
-    if (system2('dvisvgm', c('-o', shQuote(fig2), fig)) != 0)
-      stop('Failed to compile ', fig, ' to ', fig2)
+    if (system2('dvisvgm', c(
+      options$engine.opts$dvisvgm.opts, '-o', shQuote(fig2), fig
+    )) != 0) stop('Failed to compile ', fig, ' to ', fig2)
   } else {
     # convert to the desired output-format using magick
     if (ext != 'pdf') magick::image_write(do.call(magick::image_convert, c(
@@ -312,51 +391,39 @@ eng_tikz = function(options) {
   fig = fig2
 
   options$fig.num = 1L; options$fig.cur = 1L
-  extra = knit_hooks$get('plot')(fig, options)
-  options$engine = 'tex'  # for output hooks to use the correct language class
+  extra = run_hook_plot(fig, options)
   engine_output(options, options$code, '', extra)
 }
 
-## GraphViz (dot) and Asymptote are similar
-eng_dot = function(options) {
-
-  # write code to a temp file, and output to another temp file
-  f = wd_tempfile('code'); f2 = wd_tempfile('out')
-  write_utf8(code <- options$code, f)
-  on.exit(unlink(c(f, f2)), add = TRUE)
-
-  # adapt command to either graphviz or asymptote
-  if (options$engine == 'dot') {
-    command_string = '%s %s -T%s -o%s'
-    syntax         = 'dot'
-  } else if (options$engine == 'asy') {
-    command_string = '%s %s -f %s -o %s'
-    syntax         = 'cpp'  # use cpp syntax for syntax highlighting
-  }
-
-  # prepare system command
-  cmd = sprintf(
-    command_string, shQuote(get_engine_path(options$engine.path, options$engine)),
-    shQuote(f), ext <- options$fig.ext %n% dev2ext(options$dev),
-    shQuote(f2 <- paste0(f2, '.', ext))
-  )
-
-  # generate output
-  outf = paste(fig_path(), ext, sep = '.')
-  dir.create(dirname(outf), recursive = TRUE, showWarnings = FALSE)
-  unlink(outf)
-  extra = if (options$eval) {
-    if (options$message) message('running: ', cmd)
-    system(cmd)
-    file.rename(f2, outf)
-    if (!file.exists(outf)) stop('Failed to compile the ', options$engine, ' chunk')
-    options$fig.num = 1L; options$fig.cur = 1L
-    knit_hooks$get('plot')(outf, options)
-  }
-
-  # wrap
-  options$engine = syntax
-  engine_output(options, code, '', extra)
+## Commands that generate plots, e.g., GraphViz (dot), Asymptote, and Ditaa
+eng_plot = function(options) {
+  options$command = cmd = options$engine
+  options$fig.ext = ext = dev2ext(options)
+  opts = list(
+    output = function(options, code, output, file) {
+      extra = if (options$eval) {
+        # move the generated plot (with a temp filename) to fig.path
+        f1 = with_ext(file, ext)
+        f2 = paste(fig_path(), ext, sep = '.')
+        xfun::dir_create(dirname(f2))
+        unlink(f2)
+        file.rename(f1, f2)
+        options$fig.num = 1L; options$fig.cur = 1L
+        run_hook_plot(f2, options)
+      }
+      engine_output(options, code, '', extra)
+    },
+    # better default for ditaa: https://github.com/yihui/knitr/pull/2092
+    args1 = if (cmd == 'ditta') c('-s', 2, '-T', '-S', '-E'),
+    args = function(code, file) {
+      f2 = with_ext(file, ext)
+      if (cmd == 'ditaa') return(c(file, f2))
+      if (cmd %in% c('dot', 'asy')) {
+        c(file, c(dot = '-T', asy = '-f')[cmd], ext, '-o', f2)
+      }
+    })
+  options$engine.opts = merge_list(opts, options$engine.opts)
+  eng_exec(options)
 }
 
 ## Andre Simon's highlight
@@ -381,9 +448,8 @@ eng_cat = function(options) {
   if (options$eval)
     do.call(cat2, c(list(options$code), options$engine.opts))
 
-  if (is.null(lang <- options$engine.opts$lang) && is.null(lang <- options$class.source))
-    return('')
-  options$engine = lang
+  options = set_lang(options, options$class.source)
+  if (is.null(options$lang)) return('')
   engine_output(options, options$code, NULL)
 }
 
@@ -471,7 +537,7 @@ eng_block2 = function(options) {
 # helper to create engines the wrap embedded html assets (e.g. css,js)
 eng_html_asset = function(prefix, postfix) {
   function(options) {
-    out = if (options$eval && is_html_output(excludes = 'markdown')) {
+    out = if (options$eval && is_html_output()) {
       one_string(c(prefix, options$code, postfix))
     }
     options$results = 'asis'
@@ -480,7 +546,7 @@ eng_html_asset = function(prefix, postfix) {
 }
 
 # include js in a script tag (ignore if not html output)
-eng_js = eng_html_asset('<script type="text/javascript">', '</script>')
+eng_js = eng_html_asset('<script>', '</script>')
 
 # include css in a style tag (ignore if not html output)
 eng_css = eng_html_asset('<style type="text/css">', '</style>')
@@ -542,6 +608,7 @@ eng_sql = function(options) {
   if (is.na(max.print) || is.null(max.print))
     max.print = -1
   sql = one_string(options$code)
+  params = options$params
 
   query = interpolate_from_env(conn, sql)
   if (isFALSE(options$eval)) return(engine_output(options, query, ''))
@@ -557,8 +624,14 @@ eng_sql = function(options) {
       data = DBI::dbFetch(res, n = max.print)
       DBI::dbClearResult(res)
       data
+
     } else {
-      DBI::dbGetQuery(conn, query)
+      if (length(params) == 0) {
+        DBI::dbGetQuery(conn, query)
+      } else {
+        # If params option is provided, parameters are not interplolated
+        DBI::dbGetQuery(conn, sql, params = params)
+      }
     }
   }, error = function(e) {
     if (!options$error) stop(e)
@@ -587,12 +660,14 @@ eng_sql = function(options) {
       options$results = 'asis'
 
       # force left alignment if the first column is an incremental id column
-      first_column = display_data[[1]]
-      if (is.numeric(first_column) && length(first_column) > 1 && all(diff(first_column) == 1))
-        display_data[[1]] = as.character(first_column)
+      is_id = function(x) {
+        is.numeric(x) && length(x) > 1 && !anyNA(x) && all(diff(x) == 1)
+      }
+      if (is_id(display_data[[1]])) display_data[[1]] = as.character(display_data[[1]])
 
       # wrap html output in a div so special styling can be applied
-      if (is_html_output()) cat('<div class="knitsql-table">\n')
+      add_div = is_html_output() && getOption('knitr.sql.html_div', TRUE)
+      if (add_div) cat('<div class="knitsql-table">\n')
 
       # determine records caption
       caption = options$tab.cap
@@ -612,7 +687,7 @@ eng_sql = function(options) {
       print(kable(display_data, caption = caption))
 
       # terminate div
-      if (is_html_output()) cat("\n</div>\n")
+      if (add_div) cat("\n</div>\n")
 
       # otherwise use tibble if it's available
     } else if (loadable('tibble')) {
@@ -738,25 +813,114 @@ eng_sxss = function(options) {
   }
 
   engine_output(options, options$code, final_out)
+}
 
+eng_bslib = function(options) {
+  if (!loadable("bslib")) {
+    stop2("The 'bslib' package must be installed in order for the knitr engine 'bslib' to work.")
+  }
+  if (!is.null(options$engine.opts$sass_fun)) {
+    stop2("The 'bslib' knitr engine does not allow for customization of the Sass compilation function.")
+  }
+  func = sass::sass_partial
+  formals(func)$bundle = quote(bslib::bs_global_get())
+  options$engine.opts$sass_fun = func
+  eng_sxss(options)
+}
+
+# Target Markdown engine contributed by @wlandau
+# Thread: https://github.com/ropensci/targets/issues/503
+# Usage: https://books.ropensci.org/targets/markdown.html
+# Docs: https://docs.ropensci.org/targets/reference/tar_engine_knitr.html
+eng_targets = function(options) {
+  targets::tar_engine_knitr(options)
+}
+
+# an Eviews engine based on EviewsR
+eng_eviews = function(options) {
+  # EviewsR can't be installed in lower versions of R, hence I can't declare
+  # Suggests dependency in DESCRIPTION
+  f = getFromNamespace('eng_eviews', 'EviewsR')
+  f(options)
+}
+
+# a comment engine to return nothing
+eng_comment = function(options) {}
+
+# a verbatim engine that returns its chunk content verbatim
+eng_verbatim = function(options) {
+  # change default for the cat engine
+  options$eval = FALSE
+  options = set_lang(options)
+  eng_cat(options)
+}
+
+set_lang = function(options, default = 'default') {
+  # specify the lang name in engine.opts = list(lang = ), or lang/language,
+  # or class.source; if all are empty, use 'default'
+  if (is.null(options$lang)) options$lang = options$engine.opts$lang %n% default
+  options
+}
+
+# embed a file verbatim
+eng_embed = function(options) {
+  # if `file` is empty, use `code` as the list of files
+  if (is.null(f <- options$file)) {
+    f = gsub('^["\']|["\']$', '', options$code)  # in case paths are quoted
+    if (length(f) == 0) return()
+    options$code = xfun::read_all(f)
+  }
+  # use the filename extension as the default language name
+  if (nchar(lang <- file_ext(f[1])) > 1) {
+    lang = sub('^R', '', lang)  # Rmd -> md, Rhtml -> html, etc.
+    if (lang == 'nw') lang = 'tex'
+  }
+  options = set_lang(options, tolower(lang))
+  eng_verbatim(options)
 }
 
 # set engines for interpreted languages
 local({
   for (i in c(
     'awk', 'bash', 'coffee', 'gawk', 'groovy', 'haskell', 'lein', 'mysql',
-    'node', 'octave', 'perl', 'psql', 'Rscript', 'ruby', 'sas',
+    'node', 'octave', 'perl', 'php', 'psql', 'Rscript', 'ruby', 'sas',
     'scala', 'sed', 'sh', 'stata', 'zsh'
   )) knit_engines$set(setNames(list(eng_interpreted), i))
 })
 
 # additional engines
 knit_engines$set(
-  highlight = eng_highlight, Rcpp = eng_Rcpp, tikz = eng_tikz, dot = eng_dot,
-  c = eng_shlib, cc = eng_shlib, fortran = eng_shlib, fortran95 = eng_shlib, asy = eng_dot,
-  cat = eng_cat, asis = eng_asis, stan = eng_stan, block = eng_block,
-  block2 = eng_block2, js = eng_js, css = eng_css, sql = eng_sql, go = eng_go,
-  python = eng_python, julia = eng_julia, sass = eng_sxss, scss = eng_sxss
+  asis = eng_asis,
+  asy = eng_plot,
+  block = eng_block,
+  block2 = eng_block2,
+  bslib = eng_bslib,
+  c = eng_shlib,
+  cat = eng_cat,
+  cc = eng_shlib,
+  comment = eng_comment,
+  css = eng_css,
+  ditaa = eng_plot,
+  dot = eng_plot,
+  embed = eng_embed,
+  eviews = eng_eviews,
+  exec = eng_exec,
+  fortran = eng_shlib,
+  fortran95 = eng_shlib,
+  go = eng_go,
+  highlight = eng_highlight,
+  js = eng_js,
+  julia = eng_julia,
+  python = eng_python,
+  R = eng_r,
+  Rcpp = eng_Rcpp,
+  sass = eng_sxss,
+  scss = eng_sxss,
+  sql = eng_sql,
+  stan = eng_stan,
+  targets = eng_targets,
+  tikz = eng_tikz,
+  verbatim = eng_verbatim
 )
 
 cache_engines$set(python = cache_eng_python)
