@@ -204,7 +204,6 @@ format_sci_one = function(
     return(round_digits(x)) # no need sci notation
 
   b = round_digits(x / 10^lx)
-  b[b %in% c(1, -1)] = ''
 
   switch(format, latex = {
     sci_notation('%s%s10^{%s}', b, times, lx)
@@ -225,7 +224,11 @@ format_sci_one = function(
 }
 
 sci_notation = function(format, base, times, power) {
-  sprintf(format, base, ifelse(base == '', '', times), power)
+  if (base %in% c('1', '-1')) {
+    times = ''
+    base = if (base == '1') '' else '-'
+  }
+  sprintf(format, base, times, power)
 }
 
 # vectorized version of format_sci_one()
@@ -243,6 +246,33 @@ tikz_dict = function(path) {
   paste(sans_ext(basename(path)), 'tikzDictionary', sep = '-')
 }
 
+# convert dashes in option names with dots (e.g., `fig-height` to `fig.height`),
+# and vice versa (dots work for knitr, and dashes for Quarto)
+fix_names = function(x, char1, char2, aliases) {
+  name1 = grep(char1, names(x), value = TRUE, fixed = TRUE)
+  name2 = gsub(char1, char2, name1, fixed = TRUE)
+  name3 = c(names(opts_chunk_attr), names(opts_chunk$get()))
+  # only convert names that are known to knitr
+  i = (if (char1 == '.') name1 else name2) %in% name3
+  if (any(i)) {
+    x[name2[i]] = x[name1[i]]  # move values to to new names
+    x[name1[i]] = NULL  # delete old names
+  }
+  for (j in intersect(names(x), names(aliases))) {
+    x[[aliases[j]]] = x[[j]]
+    x[[j]] = NULL
+  }
+  x
+}
+
+dot_names = function(x) {
+  fix_names(x, '-', '.', c(fig.format = 'dev', fig.dpi = 'dpi'))
+}
+
+dash_names = function(x) {
+  fix_names(x, '.', '-', c(dev = 'fig-format', dpi = 'fig-dpi'))
+}
+
 # initially for compatibility with Sweave and old beta versions of knitr
 # but now also place to tweak default options
 fix_options = function(options) {
@@ -322,10 +352,6 @@ fix_options = function(options) {
   if (identical(options$strip.white, I(TRUE)))
     options$strip.white = !options$collapse
 
-  # TODO: remove this after https://github.com/DeclareDesign/randomizr/pull/95
-  if (xfun::check_old_package('randomizr', '0.22.0') && identical(options$message, 'hide'))
-    options$message = FALSE
-
   options
 }
 
@@ -797,8 +823,7 @@ has_crop_tools = function(warn = TRUE) {
   if (is_windows() && Sys.which('tlmgr') != '') {
     # assuming users know what this env var means (rstudio/tinytex#391)
     if (Sys.getenv('TEXLIVE_WINDOWS_EXTERNAL_GS') != '') return(TRUE)
-    # TODO: use tinytex::tlmgr_version('list')$year
-    year = as.integer(xfun::grep_sub('^TeX Live.* version (\\d+).*$', '\\1', tinytex::tlmgr_version())[1])
+    year = tinytex::tlmgr_version('list')$texlive
     if (year < 2023 && warn) warning(
       'TeX Live version too low. Please consider upgrading, e.g., via tinytex::reinstall_tinytex().'
     )
@@ -1116,9 +1141,7 @@ str_split = function(x, split, ...) {
 # methods to update/close it
 txt_pb = function(total, labels) {
   if (getOption('knitr.progress.linenums', FALSE)) {
-    nums = sapply(seq_along(labels), function(i) {
-      paste(current_lines(i), collapse = '-')
-    })
+    nums = sapply(seq_along(labels), current_lines)
     labels = sprintf(
       '%s%s%s:%s', labels, ifelse(labels == '', '', ' @ '),
       knit_concord$get('infile'), nums
@@ -1140,8 +1163,8 @@ txt_pb = function(total, labels) {
     if (identical(con, '')) con = stdout()
     if (!inherits(con, 'connection')) return(TRUE)
     if (isatty(con)) return(FALSE)
-    # when RStudio is available, return FALSE
-    is.null(tryCatch(rstudioapi::versionInfo(), error = function(e) NULL))
+    # when in RStudio, return FALSE
+    Sys.getenv('RSTUDIO') != '1'
   })()
   # use simple progress output without the bar but only progress and labels
   if (simple) return(list(
