@@ -22,6 +22,10 @@ op = options(digits = 4, scipen = 0)
 assert('format_sci() turns numbers into scientific notations', {
   (format_sci(c(1.84e8, 1e5, 2.34e3)) %==% c('1.84\\times 10^{8}', '10^{5}', '2340'))
   (format_sci(1.23456789 * 10^-5) %==% '1.2346\\times 10^{-5}')
+  (format_sci(10^-5) %==% '10^{-5}')
+  (format_sci(10^5) %==% '10^{5}')
+  (format_sci(-10^-5) %==% '-10^{-5}')
+  (format_sci(-10^5) %==% '-10^{5}')
   (format_sci(9.87654e6, 'html') %==% '9.8765 &times; 10<sup>6</sup>')
   (format_sci(9.87654e6, 'rst') %==% '9.8765 |times| 10 :sup:`6`')
   (format_sci(letters) %==% letters)
@@ -64,14 +68,12 @@ assert('sanitize_fn() warns against spaces in filenames', {
 options(op)
 
 assert('fig_path() sanitizes paths', {
-  (identical(sanitize_fn('fig/foo', '.png'), 'fig/foo.png'))
-  (suppressWarnings(c(
-    identical(sanitize_fn('figure/a b'), 'figure/a_b'),
-    identical(sanitize_fn('fig space/a.b'), 'fig_space/a_b'),
-    identical(sanitize_fn('../c.d'), '../c_d'),
-    identical(sanitize_fn('./../c..d'), './../c__d')
-  )))
-  (identical(sanitize_fn('C:/foo/bar'), 'C:/foo/bar'))
+  (sanitize_fn('fig/foo') %==% 'fig/foo')
+  (sanitize_fn('figure/a b  c', FALSE) %==% 'figure/a_b__c')
+  (sanitize_fn('fig space/a.b') %==% 'fig_space/a.b')
+  (sanitize_fn('../c.d') %==% '../c.d')
+  (sanitize_fn('./../c..d') %==% './../c..d')
+  (sanitize_fn('C:/foo/bar') %==% 'C:/foo/bar')
 })
 
 assert('fig_chunk() generates figure filenames for a code chunk', {
@@ -139,11 +141,11 @@ opts = list(
 assert('.img.cap() generates the figure caption and alt attribute', {
   (.img.cap(list(fig.cap = NULL), FALSE) %==% "")
   (.img.cap(opts, FALSE) %==% opts$fig.cap)
-  (.img.cap(opts, TRUE)  %==% 'Figure &quot;caption&quot; &lt;&gt;.')
+  (.img.cap(opts, TRUE, TRUE)  %==% 'Figure &quot;caption&quot; &lt;&gt;.')
 
   opts$fig.alt = 'Figure "alternative text" <>.'
 
-  (.img.cap(opts, TRUE)  %==% 'Figure &quot;alternative text&quot; &lt;&gt;.')
+  (.img.cap(opts, TRUE, TRUE)  %==% 'Figure &quot;alternative text&quot; &lt;&gt;.')
   (.img.cap(opts, FALSE)  %==% opts$fig.cap)
 
   (.img.cap(list(fig.cap = '', fig.alt = "alt"), FALSE) %==% "")
@@ -220,3 +222,59 @@ assert('pandoc_to gets the current Pandoc format', {
   (pandoc_to(c('docx', 'pptx')))
   opts_knit$set(opts)
 })
+
+assert('comment_out() add prefix and newlines if asked', {
+  (comment_out("a") %==% "## a\n")
+  (comment_out("ab cd") %==% "## ab cd\n")
+  (comment_out("ab cd\n") %==% "## ab cd\n")
+  (comment_out("") %==% "## \n")
+  (comment_out("\n") %==% "## \n")
+  (comment_out(c("a", "b")) %==% c("## a\n", "## b\n"))
+  (comment_out(c("a", "b"), which = 2) %==% c("a\n", "## b\n"))
+  (comment_out("a", newline = FALSE) %==% "## a")
+  (comment_out("a", prefix = "$") %==% "$ a\n")
+  (comment_out("a", prefix = NULL) %==% "a\n")
+})
+
+assert('remove_urls() removes the link', {
+  (remove_urls(c('[a](b)', '[a b](c)')) %==% c('a', 'a b'))
+  (remove_urls('a [b](c) d.') %==% 'a b d.')
+  (remove_urls('a [b](c) d [e f+g](h) i.') %==% 'a b d e f+g i.')
+  (remove_urls('a [b](c) `[d](e)` f.') %==% 'a b `[d](e)` f.')
+})
+
+assert('options using `-` are converted to `.` and default value replaced', {
+  opts = opts_chunk$merge(list('fig-cap' = 'caption', 'out-width' = 300))
+  (is.null(dot_names(opts)[['fig-cap']]))
+  (is.null(dot_names(opts)[['out-width']]))
+  (dot_names(opts)[['fig.cap']] %==% 'caption')
+  (dot_names(opts)[['out.width']] %==% 300)
+  rm(opts)
+})
+
+assert('fig.format and fig.dpi', {
+  opts = opts_chunk$merge(list('fig-format' = 'svg', 'fig-dpi' = 750))
+  (is.null(dot_names(opts)[['fig-format']]))
+  (is.null(dot_names(opts)[['fig-dpi']]))
+  (dot_names(opts)[['dev']] %==% 'svg')
+  (dot_names(opts)[['dpi']] %==% 750)
+  rm(opts)
+})
+
+assert('options using `.` are converted to `-` and default value replaced', {
+  opts = opts_chunk$merge(list('fig.cap' = 'caption', 'out.width' = 300))
+  (is.null(dash_names(opts)[['fig.cap']]))
+  (is.null(dash_names(opts)[['out.width']]))
+  (dash_names(opts)[['fig-cap']] %==% 'caption')
+  (dash_names(opts)[['out-width']] %==% 300)
+  rm(opts)
+})
+
+assert('dev and dpi are convert to fig-format and fig-dpi', {
+  opts = opts_chunk$merge(list('dev' = 'svg', 'dpi' = 750))
+  (is.null(dash_names(opts)[['dev']]))
+  (is.null(dash_names(opts)[['dpi']]))
+  (dash_names(opts)[['fig-format']] %==% 'svg')
+  (dash_names(opts)[['fig-dpi']] %==% 750)
+  rm(opts)
+})
